==> uima-ducc-common-2.0.0-sources.jar: org/apache/uima/ducc/common/utils/id/DuccIdFactory.java <==
/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
*/
package org.apache.uima.ducc.common.utils.id;

import org.apache.uima.ducc.common.persistence.IPropertiesFileManager;

public class DuccIdFactory implements IDuccIdFactory {
	
	private volatile long seqno = -1;

	private IPropertiesFileManager propertiesFileManager = null;
	private String propertiesFileKey = null;       

	public DuccIdFactory() {	
	}

	public DuccIdFactory(IPropertiesFileManager propertiesFileManager, String propertiesFileKey) {	
		this.propertiesFileManager = propertiesFileManager;
		this.propertiesFileKey = propertiesFileKey;
	}
	
	public DuccIdFactory(long seed) {
		seqno = seed-1;
	}
	
    /**
     * During recovery, if you pass in a "friendly", you always want the passed-in friendly, but 
     * you want to ensure that at the end of recovery, the seed is set to the largest of the
     * recovered IDs.
     */
    public DuccId next(long s) {
        seqno = Math.max(s, seqno);
        return new DuccId(s);
    }

	public DuccId next() {
		synchronized(this) {
			if(propertiesFileManager != null) {
				seqno = propertiesFileManager.increment(propertiesFileKey);
			}
			else {
				seqno++;                
			}
			return new DuccId(seqno);
		}
	}
}

==> uima-ducc-common-2.2.0-sources.jar: org/apache/uima/ducc/common/utils/id/DuccIdFactory.java <==
/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
*/
package org.apache.uima.ducc.common.utils.id;

import java.util.concurrent.atomic.AtomicLong;

import org.apache.uima.ducc.common.persistence.IPropertiesFileManager;

public class DuccIdFactory implements IDuccIdFactory {
	
	private AtomicLong seqno = new AtomicLong(-1);

	private IPropertiesFileManager propertiesFileManager = null;
	private String propertiesFileKey = null;       

	public DuccIdFactory() {	
	}

	public DuccIdFactory(IPropertiesFileManager propertiesFileManager, String propertiesFileKey) {	
		this.propertiesFileManager = propertiesFileManager;
		this.propertiesFileKey = propertiesFileKey;
	}
	
	public DuccIdFactory(long seed) {
		seqno.set(seed-1);
	}
	
    /**
     * During recovery, if you pass in a "friendly", you always want the passed-in friendly, but 
     * you want to ensure that at the end of recovery, the seed is set to the largest of the
     * recovered IDs.
     */
    public DuccId next(long s) {
        seqno.set(Math.max(s, seqno.get()));
        return new DuccId(s);
    }

	public DuccId next() {
		synchronized(this) {
			if(propertiesFileManager != null) {
				seqno.set(propertiesFileManager.increment(propertiesFileKey));
			}
			else {
				seqno.incrementAndGet();                
			}
			return new DuccId(seqno.get());
		}
	}
	
	/**
	 * set seqno to candidate if it is bigger than previous, return previous value
	 */
	public long setIfMax(long candidate) {
		long previous = seqno.get();
		synchronized(this) {
			if(propertiesFileManager != null) {
				try {
					String propertiesFileValue = propertiesFileManager.get(propertiesFileKey, ""+previous);
					previous = Long.parseLong(propertiesFileValue);
				}
				catch(Exception e) {
					// No worries
				}
				if(candidate > previous) {
					propertiesFileManager.set(propertiesFileKey, ""+candidate);
					seqno.set(candidate);
				}
			}
			else {
				if(candidate > previous) {
					seqno.set(candidate);
				}
			}
		}
		return previous;
	}
}
